@@ -276,11 +276,11 @@ int MolecularRepresentation::drawHydrogenBonds()
     if (nDots%100 != 0) {
         displayPrimitives.push_back(balls);
     }
-    
+
     mmdb->DeleteSelection(selHnd);
     colorScheme->freeSelectionHandles(mmdb, handles);
-    
-    return 0;		
+
+    return 0;
 }
 
 int MolecularRepresentation::drawBondsAsNewSticks()
@@ -288,24 +288,24 @@ int MolecularRepresentation::drawBondsAsNewSticks()
     mmdb::Manager *mmdb = myMolecule->getMmdb();
 	//selection->describe();
 	selHnd = selection->handleInMMDB(mmdb);
-    
+
     int nBonds = 0;
     std::shared_ptr<SticksPrimitive>sticks(new SticksPrimitive());
     sticks->setColorScheme(colorScheme);
     sticks->setMmdb(myMolecule->getMmdb());
-    
+
     for (int iAtom = 1; iAtom <= myMolecule->getMmdb()->GetNumberOfAtoms(); iAtom++){
         mmdb::Atom* atom1 = myMolecule->getMmdb()->GetAtomI(iAtom);
-        
+
         if (atom1->isInSelection(selHnd)){
             int nBondedAtoms;
             mmdb::AtomBond * bondedAtoms;
             atom1->GetBonds(bondedAtoms, nBondedAtoms);
             for (int iOtherAtom = 0; iOtherAtom < nBondedAtoms; iOtherAtom++){
                 mmdb::Atom* atom2 = bondedAtoms[iOtherAtom].atom;
-                if (atom2->GetIndex()>iAtom && 
+                if (atom2->GetIndex()>iAtom &&
                     atom2->isInSelection(selHnd)){
-                    
+
                     //Nasty kludge here...MMDB's MakeBonds screws up where there are multiple conformations
                     if (!strcmp(atom1->altLoc,"") &&
                         !strcmp(atom2->altLoc,"")) {
@@ -322,7 +322,7 @@ int MolecularRepresentation::drawBondsAsNewSticks()
                             nBonds++;
                         }
                     }
-                    
+
                     if (nBonds%1000 == 0) {
                         displayPrimitives.push_back(sticks);
                         sticks = std::shared_ptr<SticksPrimitive>(new SticksPrimitive());
@@ -337,8 +337,7 @@ int MolecularRepresentation::drawBondsAsNewSticks()
         displayPrimitives.push_back(sticks);
     }
     mmdb->DeleteSelection(selHnd);
-	
-    return 0;		
+    return 0;
 }
 
 int MolecularRepresentation::drawDishyBases()
@@ -352,36 +351,36 @@ int MolecularRepresentation::drawDishyBases()
     FCXXCoord xAxis (1., 0., 0., 0.);
     FCXXCoord yAxis (0., 1., 0., 0.);
     FCXXCoord zAxis (0., 0., 1., 0.);
-    
+
     std::shared_ptr<BallsPrimitive>balls(new BallsPrimitive());
 
     shared_ptr<CylindersPrimitive>cylinder(new CylindersPrimitive());
-    cylinder->setAngularSampling(intParameters["cylindersStyleAngularSampling"]);
-    
+    cylinder->setAngularSampling(intParameters["dishStyleAngularSampling"]);
+
     float cylinderRadius = floatParameters[std::string("cylindersStyleCylinderRadius")];
     float ballRadius = floatParameters[std::string("cylindersStyleBallRadius")];
-  
+
     auto dbContainerIter = dishy_bases_chain_map.begin();
     std::map<std::shared_ptr<ColorRule>,int>handles = colorScheme->prepareForMMDB(mmdb);
-    
+
     for (; dbContainerIter!=dishy_bases_chain_map.end();dbContainerIter++){
         auto dbCont = dbContainerIter->second;
         auto dishyBaseIter = (dbContainerIter->second).bases.begin();
         for (; dishyBaseIter != (dbContainerIter->second).bases.end(); ++dishyBaseIter){
-            
+
             auto atom1 = dishyBaseIter->ribose_atoms[1];
             FCXXCoord atom1Color =  colorScheme->colorForAtom(atom1, handles);
-            
+
             balls->addBall(dishyBaseIter->centre, atom1Color, dishyBaseIter->radius, dishyBaseIter->normal, 0.14*dishyBaseIter->radius);
-            
+
             if (balls->getBalls().size()%100 == 0){
                 displayPrimitives.push_back(balls);
                 balls = std::shared_ptr<BallsPrimitive>(new BallsPrimitive);
             }
-            
+
             auto flatFanPrimitive = std::shared_ptr<FlatFanPrimitive>(new FlatFanPrimitive(dishyBaseIter->ribose_atoms, atom1Color));
             displayPrimitives.push_back(flatFanPrimitive);
-            
+
             auto riboseAtomIter = dishyBaseIter->ribose_atoms.begin();
             for (; riboseAtomIter!= dishyBaseIter->ribose_atoms.end(); ++ riboseAtomIter){
                 FCXXCoord coord((*riboseAtomIter)->x, (*riboseAtomIter)->y, (*riboseAtomIter)->z);
@@ -423,6 +422,103 @@ int MolecularRepresentation::drawDishyBases()
     return 0;
 }
 
+int MolecularRepresentation::drawStickBases() {
+
+   auto is_nucleic_acid = [] (const std::string &base_name) {
+      if (base_name == "G") return true;
+      if (base_name == "A") return true;
+      if (base_name == "T") return true;
+      if (base_name == "C") return true;
+      if (base_name == "U") return true;
+      if (base_name == "DG") return true;
+      if (base_name == "DA") return true;
+      if (base_name == "DC") return true;
+      if (base_name == "DT") return true;
+      // others
+      return false;
+   };
+
+   auto get_atom_name_pair = [] (const std::string &base_name) {
+      if (base_name == "DG" || base_name == "DA" || base_name == "G" || base_name == "A")
+         return std::pair<std::string, std::string> (" C3'", " N1 ");
+      if (base_name == "DT" || base_name == "U")
+         return std::pair<std::string, std::string> (" C3'", " O4 ");
+      if (base_name == "DC" || base_name == "C")
+         return std::pair<std::string, std::string> (" C3'", " N4 ");
+      return std::pair<std::string, std::string> ("", "");
+   };
+
+   mmdb::Manager *mmdb = myMolecule->getMmdb();
+   selHnd = selection->handleInMMDB(mmdb);
+   shared_ptr<CylindersPrimitive>cylinder(new CylindersPrimitive());
+   cylinder->setAngularSampling(intParameters["cylindersStyleAngularSampling"]);
+   float cylinderRadius = floatParameters[std::string("cylindersStyleCylinderRadius")];
+   std::map<std::shared_ptr<ColorRule>,int>handles = colorScheme->prepareForMMDB(mmdb);
+   float ballRadius = floatParameters[std::string("cylindersStyleBallRadius")];
+   std::shared_ptr<BallsPrimitive>balls(new BallsPrimitive());
+
+   {
+      for(int imod = 1; imod<=mmdb->GetNumberOfModels(); imod++) {
+         mmdb::Model *model_p = mmdb->GetModel(imod);
+         if (model_p) {
+            int n_chains = model_p->GetNumberOfChains();
+            for (int ichain=0; ichain<n_chains; ichain++) {
+               mmdb::Chain *chain_p = model_p->GetChain(ichain);
+               int n_res = chain_p->GetNumberOfResidues();
+               for (int ires=0; ires<n_res; ires++) {
+                  mmdb::Residue *residue_p = chain_p->GetResidue(ires);
+                  if (residue_p) {
+                     if (residue_p->isDNARNA()) {
+                        std::string base_name(residue_p->GetResName());
+                        if (is_nucleic_acid(base_name)) {
+
+                           std::pair<std::string, std::string> atom_name_pair = get_atom_name_pair(base_name);
+                           const std::string atom_name_1 = atom_name_pair.first;
+                           const std::string atom_name_2 = atom_name_pair.second;
+                           mmdb::Atom *atom_1 = nullptr;
+                           mmdb::Atom *atom_2 = nullptr;
+
+                           // iterate over all alt-confs in the residue
+                           // (which I am not doing at the moment)
+
+                           mmdb::PPAtom residue_atoms = nullptr;
+                           int nResidueAtoms = 0;
+                           residue_p->GetAtomTable(residue_atoms, nResidueAtoms);
+                           for(int i = 0; i < nResidueAtoms; i++) {
+                              mmdb::Atom *at = residue_atoms[i];
+                              std::string atom_name(at->GetAtomName());
+                              if (! atom_1)
+                                 if (atom_name == atom_name_1)
+                                    atom_1 = at;
+                              if (! atom_2)
+                                 if (atom_name == atom_name_2)
+                                    atom_2 = at;
+                           }
+                           if (atom_1 && atom_2) {
+                               if(atom_1->isInSelection(selHnd)&&atom_2->isInSelection(selHnd)) {
+                                   FCXXCoord atom1Color = colorScheme->colorForAtom(atom_1, handles);
+                                   FCXXCoord atom2Color = colorScheme->colorForAtom(atom_2, handles);
+                                   cylinder->addHalfAtomBond(atom_1, atom1Color, atom_2, atom2Color, cylinderRadius);
+                                   FCXXCoord atom1Coord(atom_2->x,atom_2->y, atom_2->z);
+                                   balls->addBall(atom1Coord, atom1Color, ballRadius);
+                              }
+                           }
+                        }
+                     }
+                  }
+               }
+            }
+         }
+      }
+   }
+
+   mmdb->DeleteSelection(selHnd);
+   displayPrimitives.push_back(cylinder);
+   displayPrimitives.push_back(balls);
+   colorScheme->freeSelectionHandles(mmdb, handles);
+   return 0;
+}
+
 int MolecularRepresentation::drawRibbon()
 {
     int subdivisionsPerCalpha = intParameters["ribbonStyleAxialSampling"];
@@ -447,7 +543,7 @@ int MolecularRepresentation::drawRibbon()
     std::vector<DiscreteSegment *> segments;
     myMolecule->identifySegments(segments, selHnd);
 
-    for (int iSegment = 0; iSegment < segments.size(); iSegment++){
+    for (std::size_t iSegment = 0; iSegment < segments.size(); iSegment++){
         DiscreteSegment &segment = *(segments[iSegment]);
         segment.evaluateNormals();
         if (boolParameters["smoothBetas"]) segment.smoothBetas();
@@ -469,7 +565,7 @@ int MolecularRepresentation::drawRibbon()
             else currentSSE = const_cast<mmdb::Atom*>(calpha)->GetResidue()->SSE;
             //std::cout << const_cast<mmdb::Atom*>(calpha)->GetResidue()->GetResidueNo() <<":"<<currentSSE<<std::endl;
             if (currentSSE == mmdb::SSE_Bulge) currentSSE = mmdb::SSE_Strand;
-            
+
             int nextSSE = mmdb::SSE_None;
             if (iCalpha < segment.nCalphas()-1){
                 auto nextResidue = segment.calpha(iCalpha+1)->GetResidue();
@@ -491,11 +587,11 @@ int MolecularRepresentation::drawRibbon()
                 }
             }
             color = colorScheme->colorForAtom(calpha, handles);
-            
+
             int endSubdivision = ((iCalpha == (segment.nCalphas()-1))?subdivisionsPerCalpha/2:subdivisionsPerCalpha);
             int startSubdivision = ((iCalpha == 0) ? (subdivisionsPerCalpha/2):0);
             float xVal = 0.;
-            
+
             for (int i=startSubdivision; i<endSubdivision; i++){
                 //for (int i=0; i<subdivisionsPerCalpha; i++){
                 xVal = (iCalpha + (i*stepPerSubdivision)) - 0.5;
@@ -595,13 +691,13 @@ int MolecularRepresentation::drawRibbon()
                 }
             }
             lastSSE = currentSSE;
-            
+
             //Add an extra segment to delimit the end of a residue
             xVal += stepPerSubdivision;
             FCXXCoord coord = segment.coordFor(xVal);
             FCXXCoord normalOne = segment.normalOneFor(xVal);
             FCXXCoord normalTwo = segment.normalTwoFor(xVal);
-            
+
             float radiusOne;
             float radiusTwo;
             radiusOne = radiusOneNone;
@@ -621,15 +717,15 @@ int MolecularRepresentation::drawRibbon()
             CylinderPoint cylinderPoint(coord, color, normalOne, normalTwo, radiusOne, radiusTwo, calpha);
             if (currentSSE == mmdb::SSE_Strand) currentBoxSection->addPoint(cylinderPoint);
             else currentCylinder->addPoint(cylinderPoint);
-        }        
+        }
         if (currentSSE == mmdb::SSE_Strand) displayPrimitives.push_back(currentBoxSection);
         else if (currentSSE != -32767) displayPrimitives.push_back(currentCylinder);
     }
     colorScheme->freeSelectionHandles(mmdb, handles);
     mmdb->DeleteSelection(selHnd);
-	
+
     return 0;
-}   
+}
 
 int MolecularRepresentation::drawCalphas()
 {
@@ -637,17 +733,17 @@ int MolecularRepresentation::drawCalphas()
 	//selection->describe();
 	selHnd = selection->handleInMMDB(mmdb);
     std::map<std::shared_ptr<ColorRule>,int>handles = colorScheme->prepareForMMDB(mmdb);
-    
+
     std::vector<DiscreteSegment *> segments;
     myMolecule->identifySegments(segments, selHnd);
-    
+
     std::shared_ptr<BondsPrimitive>bonds(new BondsPrimitive());
     bonds->setColorScheme(colorScheme);
     displayPrimitives.push_back(bonds);
-    
-    for (int iSegment = 0; iSegment < segments.size(); iSegment++){
+
+    for (std::size_t iSegment = 0; iSegment < segments.size(); iSegment++){
         DiscreteSegment &segment = *(segments[iSegment]);
-        
+
         for (int i=0; i<(segment.nCalphas()-1); i++){
             mmdb::Atom* atom1 = segment.calpha(i);
             mmdb::Atom* atom2 = segment.calpha(i+1);
@@ -656,12 +752,12 @@ int MolecularRepresentation::drawCalphas()
         }
     }
     bonds->evaluateGLPrimitives(handles);
-    
+
     colorScheme->freeSelectionHandles(mmdb, handles);
     mmdb->DeleteSelection(selHnd);
-    
+
     return 0;
-}    
+}
 
 int MolecularRepresentation::drawMolecularSurface()
 {
@@ -671,26 +767,27 @@ int MolecularRepresentation::drawMolecularSurface()
 int MolecularRepresentation::drawSurfaceOfKind(int surfaceKind)
 {
     //return 0;
-    
+
     mmdb::Manager *mmdb = myMolecule->getMmdb();
 	//selection->describe();
 	selHnd = selection->handleInMMDB(mmdb);
 	std::map<std::shared_ptr<ColorRule>,int>handles = colorScheme->prepareForMMDB(mmdb);
-    
-    //If we are limited to short int for indexes, we wil have to chop this surfaace
-    //into bite-sized chunks    
+
+    //If we are limited to short int for indexes, we wil have to chop this surface
+    //into bite-sized chunks
     mmdb::Atom** SelAtoms;
     int nAtoms;
-    
+
     int chunkSize = 1000000;
     if (sizeof(GLIndexType) == sizeof(short)) chunkSize = 100;
     mmdb->GetSelIndex(selHnd, SelAtoms, nAtoms);
+
     int iAtom = 0;
     int chunkHndl = mmdb->NewSelection();
-    
+
     float probeRadius = floatParameters["surfaceStyleProbeRadius"];
     float radiusMultiplier = floatParameters["ballsStyleRadiusMultiplier"];
-    
+
     for (; iAtom<nAtoms; iAtom++){
         mmdb->SelectAtom(chunkHndl, SelAtoms[iAtom], mmdb::SKEY_OR);
         if ((iAtom+1)%chunkSize == 0){ 
@@ -703,7 +800,7 @@ int MolecularRepresentation::drawSurfaceOfKind(int surfaceKind)
             chunkHndl = mmdb->NewSelection();
         }
         if (redrawProgressCallback){
-            redrawProgressCallback(redrawProgressCallbackUserInfo, (float)iAtom / (float)nAtoms);   
+            redrawProgressCallback(redrawProgressCallbackUserInfo, (float)iAtom / (float)nAtoms);
         }
 
     }
@@ -714,10 +811,10 @@ int MolecularRepresentation::drawSurfaceOfKind(int surfaceKind)
         }
         mmdb->DeleteSelection(chunkHndl);
     }
-    
+
     colorScheme->freeSelectionHandles(mmdb, handles);
     mmdb->DeleteSelection(selHnd);
-    
+
     return 0;
 }
 
@@ -754,7 +851,7 @@ void MolecularRepresentation::colorByPotential(std::string chargingAtomString, s
     auto theClipperNXMap = theCreator->coerceToClipperMap(cell);
     // Now bring the surface and the map together
     double coords[4];
-    
+
     //Create a color ramp rule
     AtomPropertyRampColorRule rampRule;
     rampRule.setStartRGB(FCXXCoord (1.,0.,0.,1.));
@@ -762,7 +859,7 @@ void MolecularRepresentation::colorByPotential(std::string chargingAtomString, s
     rampRule.setMiddleRGB(FCXXCoord (1.,1.,1.,1.));
     rampRule.setEndRGB(FCXXCoord (0.,0.,1.,1.));
     rampRule.setEndValue( 0.5);
-    
+
     auto primitivePntr = getDisplayPrimitives().begin();
     for (; primitivePntr != getDisplayPrimitives().end(); ++primitivePntr){
         if (VertexColorNormalPrimitive *p = dynamic_cast<VertexColorNormalPrimitive *>(primitivePntr->get())){
